#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

int main(int argc, char *argv[], char *envp[])
{
    struct timespec t0, t1;
    pid_t pid;
    int status;
    int ret;

    if (argc < 2) {
        fprintf(stderr, "Usage: mytime <program> [<arg> [<...>]]\n");
        return EXIT_FAILURE;
    }

    ret = clock_gettime(CLOCK_MONOTONIC, &t0);
    if (ret == -1) {
        perror("clock_gettime");
        return EXIT_FAILURE;
    }

    pid = fork();
    if (pid == -1) {
        perror("fork");
        return EXIT_FAILURE;
    }
    else if (pid == 0) {
        /* child -- run program */
        execvpe(argv[1], &argv[1], envp);
        perror("execve");
        return EXIT_FAILURE;
    }

    /* parent -- wait for program to finish */
    ret = waitpid(pid, &status, 0);
    if (ret == -1) {
        perror("waitpid");
        return EXIT_FAILURE;
    }

    ret = clock_gettime(CLOCK_MONOTONIC, &t1);
    if (ret == -1) {
        perror("clock_gettime");
        return EXIT_FAILURE;
    }

    printf("Process %d returned status %d after %ld us\n", pid, WEXITSTATUS(status),
           (t1.tv_sec - t0.tv_sec) * 1000000 + (t1.tv_nsec - t0.tv_nsec) / 1000);

    return EXIT_SUCCESS;
}
